// ======================================================================
//
// MonoDebug.h
// jeff grills
//
// copyright 1997 origin systems inc
//
// ======================================================================

#include "First.h"
#include "MonoDebug.h"

#include "Os.h"

// ======================================================================

#ifndef NMONO

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>

// ======================================================================

const int MONO_VERSION           = 0x00020004;

const int MONO_API_VERSION       = 1;
const int MONO_API_INIT          = 2;
const int MONO_API_CLEAR         = 3;
const int MONO_API_CLEARTOCURSOR = 4;
const int MONO_API_SETCURSOR     = 5;
const int MONO_API_GETCURSOR     = 6;
const int MONO_API_PUTCHAR       = 7;
const int MONO_API_GETCHAR       = 8;
const int MONO_API_PRINT         = 9;
const int MONO_API_COPYTO        = 10;
const int MONO_API_COPYFROM      = 11;

// ======================================================================

HANDLE MonoDebug::VxD;
int    MonoDebug::installed;

// ======================================================================

void MonoDebug::install(void)
{
	static char buffer[32];
	int         version;

	// Dynamically load and prepare to call MonoDebg VxD
	VxD = CreateFile("\\\\.\\MONODEBG.VXD", 0, 0, 0, CREATE_ALWAYS, FILE_FLAG_DELETE_ON_CLOSE, 0);
	if (VxD == INVALID_HANDLE_VALUE)
		return;

	// attempt to detect a mono video adapter
#if _MSC_VER >= 1100
	_outp(0x3b4, 0x0e);
	version = _inp(0x3b5);
#elif defined(__WATCOMC__)
	outp(0x3b4, 0x0e);
	version = inp(0x3b5);
#else
#error "MonoDebug::install bad compiler"
#endif

	if (version == 0xff)
	{
		OutputDebugString("Mono card not detected\n");
		CloseHandle(VxD);
		return;
	}
	else
		OutputDebugString("Mono card detected\n");

	// clear the mono screen
	if (!DeviceIoControl(VxD, MONO_API_VERSION, (LPVOID)NULL, 0, (LPVOID)&version, sizeof(version), NULL, NULL))
	{
		CloseHandle(VxD);
		return;
	}

	if (version != MONO_VERSION)
	{
		CloseHandle(VxD);
		sprintf(buffer, "MonoDebug::install expecting version %08x, got %08x\n", MONO_VERSION, version);
		OutputDebugString(buffer);
		return;
	}

	if (!DeviceIoControl(VxD, MONO_API_INIT, (LPVOID)NULL, 0, (LPVOID)NULL, 0, NULL, NULL))
	{
		CloseHandle(VxD);
		OutputDebugString("MonoDebug::install init failed\n");
		return;
	}

	installed = 1;

	sprintf(buffer, "MonoDebg.VxD version %08x\n", version);
	OutputDebugString(buffer);
	print(buffer);
}

// ----------------------------------------------------------------------

void MonoDebug::remove(void)
{
	// dynamically unload the MonoDebg VxD
	if (installed)
	{
		installed = 0;
		CloseHandle(VxD);
	}
}

// ----------------------------------------------------------------------

void MonoDebug::clear(void)
{
	if (!DeviceIoControl(VxD, MONO_API_CLEAR, (LPVOID)NULL, 0, (LPVOID)NULL, 0, NULL, NULL))
		remove();
}

// ----------------------------------------------------------------------

void MonoDebug::clearToCursor(void)
{
	if (!DeviceIoControl(VxD, MONO_API_CLEARTOCURSOR, (LPVOID)NULL, 0, (LPVOID)NULL, 0, NULL, NULL))
		remove();
}

// ----------------------------------------------------------------------

void MonoDebug::setCursor(int x, int y)
{
	int xy[2];

	xy[0] = x;
	xy[1] = y;

	if (!DeviceIoControl(VxD, MONO_API_SETCURSOR, (LPVOID)xy, sizeof(xy), (LPVOID)NULL, 0, NULL, NULL))
		remove();
}

// ----------------------------------------------------------------------

void MonoDebug::copyTo(const char *scrn)
{
	if (!DeviceIoControl(VxD, MONO_API_COPYTO, (LPVOID)scrn, 4000, (LPVOID)NULL, 0, NULL, NULL))
		remove();
}

// ----------------------------------------------------------------------

void MonoDebug::copyFrom(char *scrn)
{
	if (!DeviceIoControl(VxD, MONO_API_COPYFROM, (LPVOID)NULL, 0, (LPVOID)scrn, 4000, NULL, NULL))
		remove();
}

// ----------------------------------------------------------------------

void MonoDebug::print(const char *str)
{
	if (!DeviceIoControl(VxD, MONO_API_PRINT, (LPVOID)str, 4000, (LPVOID)NULL, 0, NULL, NULL))
	{
		DEBUG_FATAL(1, ("MonoDebug::print failed (buffer possibly on stack?!)"));
		remove();
	}
}

// ======================================================================

#endif
